<?php

/**
 * @file
 * This module uses the question interface.
 *
 * Quiz voice answer module provides a new content type to use with quiz module
 * as a question type. It allows users to record and save voice message as a
 * response to quiz question.
 */

/**
 * Extension of QuizQuestion.
 */
class QuizVoiceAnswerQuestion extends QuizQuestion {

  /**
   * Implementation of validateNode.
   *
   * This abstract function.
   *
   * @see QuizQuestion::validateNode($form)
   */
  public function validateNode(array &$form) {

  }

  /**
   * Implementation of saveNodeProperties.
   *
   * This abstract function.
   *
   * @see QuizQuestion::saveNodeProperties($is_new)
   */
  public function saveNodeProperties($is_new = FALSE) {

  }

  /**
   * Implementation of getAnsweringForm.
   */
  public function getAnsweringForm(array $form_state = NULL, $result_id) {
    $element = parent::getAnsweringForm($form_state, $result_id);
    global $user;
    if (libraries_get_path('Recordmp3js-Drupal')) {
      libraries_load('Recordmp3js-Drupal');
    }
    else {
      $download_url = 'https://github.com/Ruslan03492/Recordmp3js-Drupal';
      $args = array(
        '!url' => l($download_url, $download_url),
      );
      $msg = t('Please download the Recordmp3js-Drupal library from !url.', $args);
      drupal_set_message($msg, 'warning');
      return array();
    }
    $element['actions'] = array(
      '#type' => 'fieldset',
      '#title' => t('Recording answer'),
    );
    $settings = array(
      'nid' => $this->node->nid,
      'uid' => $user->uid,
    );
    $element['actions']['#attached']['js'][] = array(
      'data' => array('quiz_voice_answer' => $settings),
      'type' => 'setting',
    );
    $element['actions']['rec'] = array(
      '#theme' => 'html_tag',
      '#tag' => 'input',
      '#attributes' => array(
        'id' => 'rec',
        'type' => 'button',
        'value' => t('Rec'),
      ),
    );
    $element['actions']['stop'] = array(
      '#theme' => 'html_tag',
      '#tag' => 'input',
      '#attributes' => array(
        'id' => 'stop',
        'type' => 'button',
        'disabled' => 'disabled',
        'value' => t('Stop'),
      ),
    );
    $element['actions']['log'] = array(
      '#theme' => 'html_tag',
      '#tag' => 'pre',
      '#attributes' => array(
        'id' => 'log',
      ),
      '#value' => '',
    );
    $element['actions']['recordingslist'] = array(
      '#theme' => 'html_tag',
      '#tag' => 'div',
      '#attributes' => array(
        'id' => 'recordingslist',
      ),
      '#value' => '',
    );
    $element['tries'] = array(
      '#type' => 'hidden',
      '#default_value' => '',
      '#attributes' => array('id' => 'tries'),
    );
    if (isset($result_id)) {
      $record_response = new QuizVoiceAnswerResponse($result_id, $this->node);
      $response = $record_response->getResponse();
      if (is_numeric($response)) {
        $file = file_load($response);
        $out = file_view($file);
        $element['actions']['recordingslist']['#value'] = drupal_render($out);
        $element['tries']['#default_value'] = $response;
      }
    }
    return $element;
  }

  /**
   * Question response validator.
   */
  public function getAnsweringFormValidate(array &$element, &$value) {
    if ($value == '') {
      form_error($element, t('You must provide an answer.'));
    }
  }

  /**
   * Implementation of getBodyFieldTitle.
   *
   * @see QuizQuestion::getBodyFieldTitle()
   */
  public function getBodyFieldTitle() {
    return t('Question');
  }

  /**
   * Implementation of getCreationForm.
   *
   * @see QuizQuestion::getCreationForm($form_state)
   */
  public function getCreationForm(array &$form_state = NULL) {
    return array();
  }

  /**
   * Implementation of getMaximumScore.
   *
   * @see QuizQuestion::getMaximumScore()
   */
  public function getMaximumScore() {
    return variable_get('quiz_voice_answer_default_max_score', 10);
  }

}

/**
 * Extension of QuizQuestionResponse.
 */
class QuizVoiceAnswerResponse extends QuizQuestionResponse {
  /**
   * ID of the answer.
   */
  protected $answer_id = 0;
  private $answer_feedback;
  private $answer_feedback_format;

  /**
   * Constructor.
   */
  public function __construct($result_id, stdClass $question_node, $answer = NULL) {
    parent::__construct($result_id, $question_node, $answer);
    if (!isset($answer) || is_numeric($answer)) {
      $r = db_select('quiz_voice_answer_user_answers', 'qva')
        ->fields('qva', array(
          'answer_id',
          'answer',
          'score',
          'is_evaluated',
          'question_vid',
          'answer_feedback',
        ))
        ->condition('qva.question_nid', $question_node->nid)
        ->condition('qva.question_vid', $question_node->vid)
        ->condition('qva.result_id', $result_id)
        ->execute()
        ->fetch();
      if (!empty($r)) {
        $this->answer = $r->answer;
        $this->score = $r->score;
        $this->evaluated = $r->is_evaluated;
        $this->answer_id = $r->answer_id;
        $this->answer_feedback = $r->answer_feedback;
        $this->answer_feedback_format = filter_default_format();
      }
    }
    else {
      if ($file = $this->upload($answer, $this->question->nid)) {
        $this->answer = $file->fid;
        $this->evaluated = FALSE;
      }
    }
  }

  /**
   * Upload file or get isset file.
   *
   * @param string $data
   *   Data audio or fid file.
   *
   * @return array
   *   An array of variables file.
   */
  public function upload($data = NULL, $nid = 0) {
    if (!empty($data['tries'])) {
      global $user;
      $data = $data['tries'];
      list(, $data) = explode(';', $data);
      list(, $data) = explode(',', $data);
      $data = base64_decode($data);
      $path_upload = 'public://quiz_voice_answer/' . $user->uid;
      if (!file_exists($path_upload)) {
        drupal_mkdir($path_upload, NULL, TRUE);
      }
      if (file_exists($path_upload)) {
        $file = file_save_data(
          $data, $path_upload . '/audio_recording_' . $nid . '_'
          . $user->uid . '.mp3'
        );
        if ($file) {
          return $file;
        }
      }
    }
    return NULL;
  }

  /**
   * Implementation of save.
   *
   * @see QuizQuestionResponse::save()
   */
  public function save() {
    $this->answer_id = db_insert('quiz_voice_answer_user_answers')
      ->fields(
        array(
          'answer' => $this->answer,
          'question_nid' => $this->question->nid,
          'question_vid' => $this->question->vid,
          'result_id' => $this->result_id,
          'score' => $this->getScore(),
        )
      )
      ->execute();
  }

  /**
   * Implementation of delete.
   *
   * @see QuizQuestionResponse::delete()
   */
  public function delete() {
    db_delete('quiz_voice_answer_user_answers')
      ->condition('question_nid', $this->question->nid)
      ->condition('question_vid', $this->question->vid)
      ->condition('result_id', $this->result_id)
      ->execute();
  }

  /**
   * Implementation of score.
   *
   * @see QuizQuestionResponse::score()
   */
  public function score() {
    return (int) db_select('quiz_voice_answer_user_answers', 'qva')
      ->fields('qva', array('score'))
      ->condition('qva.question_nid', $this->question->nid)
      ->condition('qva.question_vid', $this->question->vid)
      ->condition('qva.result_id', $this->result_id)
      ->execute()
      ->fetchField();
  }

  /**
   * Implementation of isCorrect.
   *
   * @see QuizQuestionResponse::isCorrect()
   */
  public function isCorrect() {
    return TRUE;
  }

  /**
   * Implementation of getResponse.
   *
   * @see QuizQuestionResponse::getResponse()
   */
  public function getResponse() {
    return $this->answer;
  }

  /**
   * Implementation of getReportFormResponse
   */
  public function getFeedbackValues() {
    $data = array();
    $out = '';
    if ($file = file_load($this->answer)) {
      $out = file_view($file);
    }
    $data[] = array(
      // Hide this column. Does not make sense for long answer as there are no choices.
      'choice' => NULL,
      'attempt' => drupal_render($out),
      'correct' => $this->question->answers[0]['is_correct'] ? quiz_icon('correct') : quiz_icon(''),
      'score' => !$this->evaluated ? t('This answer has not yet been scored.') : $this->getScore(),
      'answer_feedback' => check_markup($this->answer_feedback, $this->answer_feedback_format),
    );

    return $data;
  }

  /**
   * Implementation of getReportFormAnswerFeedback.
   */
  public function getReportFormAnswerFeedback() {
    return array(
      '#title' => t('Enter feedback'),
      '#type' => 'text_format',
      '#default_value' => isset($this->answer_feedback) ? check_markup($this->answer_feedback, filter_default_format()) : '',
      '#format' => isset($this->answer_feedback_format) ? $this->answer_feedback_format : filter_default_format(),
      '#attributes' => array('class' => array('quiz-report-score')),
    );
  }

  /**
   * Implementation of getReportFormSubmit.
   */
  public function getReportFormSubmit() {
    return 'quiz_voice_answer_report_submit';
  }

  /**
   * Implementation of getReportFormValidate.
   */
  public function getReportFormValidate(&$element, &$form_state) {
    $max = $this->question->max_score;
    // Check to make sure that entered score is not higher than max allowed score.
    if ($element['score']['#value'] > $max) {
      form_error($element['score'], t('The score needs to be a number between @min and @max', array('@min' => 0, '@max' => $max)));
    }
  }

}
